module Legion
  module Extensions
    module Actors
      class Subscription
        include Celluloid::IO
        finalizer :cancel

        def initialize
          require class_path
          @queue = queue.new
          async.subscribe
        end

        def cancel
          Legion::Logging.debug "Closing subscription to #{@queue.name}"
          @queue.close
        end

        def action(payload)
          Legion::Logging.warn "Payload from default action was #{payload}"
        end

        def subscribe(manual_ack = true)
          require 'legion/extensions/tasker/runners/task_updater'
          @queue.subscribe(manual_ack: manual_ack) do |delivery_info, _metadata, payload|
            begin
              message = Legion::JSON.load(payload)
              Legion::Runner::Runner.new(runner_class, runner_method, message)
              @queue.acknowledge(delivery_info.delivery_tag) if manual_ack
            rescue StandardError => ex
              Legion::Logging.error(ex.message)
              Legion::Logging.error(ex.backtrace)
              @queue.reject(delivery_info.delivery_tag) if manual_ack
            end
          end
        end
      end
    end
  end
end
