module Legion
  module Extensions
    module Default
      def log
        if @Logger.nil?
          @logger = Legion::Logging::Logger.new
          @logger.setup
        end
        @logger
      end

      def actors
        []
      end

      def requirements
        ['legion-transport']
      end

      def lex_methods
        []
      end

      def default_settings
        {
          workers:   1,
          interval:  10,
          autobuild: {
            transport: true
          }
        }
      end

      def autobuild
        autobuild_transport if requirements.include?('legion-transport')
        autobuild_data if requirements.include?('legion-data')
        autobuild_cache if requirements.include?('legion-cache')
      end

      def autobuild_transport
        require "#{path}transport/autobuild.rb"
        klass::Transport::AutoBuild.build
      rescue StandardError => ex
        Legion::Logging.error "#{klass}::Transport::AutoBuild failed"
        Legion::Logging.debug ex.message
        Legion::Logging.debug ex.backtrace
        raise ex
      end

      def autobuild_data
        require "#{path}data/autobuild.rb"
        klass::Transport::AutoBuild.build
      rescue StandardError => ex
        Legion::Logging.error "#{klass}::Data::AutoBuild failed"
        Legion::Logging.debug ex.message
        Legion::Logging.debug ex.backtrace
        raise ex
      end

      def autobuild_cache
        require "#{path}cache/autobuild.rb"
        klass::Cache::AutoBuild.build
      rescue StandardError => ex
        Legion::Logging.error "#{klass}::Cache::AutoBuild failed"
        Legion::Logging.debug ex.message
        Legion::Logging.debug ex.backtrace
        raise ex
      end
    end
  end
end
